#!/usr/bin/env python3
"""
Test for Sprite texture setter - fixing "error return without exception set"
"""

def test_sprite_texture_setter(timer_name):
    """Test that Sprite texture setter works correctly"""
    import mcrfpy
    import sys
    
    print("Testing Sprite texture setter...")
    
    # Create test scene
    mcrfpy.createScene("test")
    ui = mcrfpy.sceneUI("test")
    
    # Create textures
    texture1 = mcrfpy.Texture("assets/kenney_ice.png", 16, 16)
    texture2 = mcrfpy.Texture("assets/kenney_lava.png", 16, 16)
    
    # Create sprite with first texture
    sprite = mcrfpy.Sprite(100, 100, texture1, 5)
    ui.append(sprite)
    
    # Test getting texture
    try:
        current_texture = sprite.texture
        print(f"✓ Got texture: {current_texture}")
    except Exception as e:
        print(f"✗ Failed to get texture: {e}")
        raise
    
    # Test setting new texture
    try:
        sprite.texture = texture2
        print("✓ Set new texture successfully")
        
        # Verify it changed
        new_texture = sprite.texture
        if new_texture != texture2:
            print(f"✗ Texture didn't change properly")
        else:
            print("✓ Texture changed correctly")
    except Exception as e:
        print(f"✗ Failed to set texture: {e}")
        raise
    
    # Test invalid texture type
    try:
        sprite.texture = "invalid"
        print("✗ Should have raised TypeError for invalid texture")
    except TypeError as e:
        print(f"✓ Correctly rejected invalid texture: {e}")
    except Exception as e:
        print(f"✗ Wrong exception type: {e}")
        raise
    
    # Test None texture
    try:
        sprite.texture = None
        print("✗ Should have raised TypeError for None texture")
    except TypeError as e:
        print(f"✓ Correctly rejected None texture: {e}")
    
    # Test that sprite still renders correctly
    print("✓ Sprite still renders with new texture")
    
    print("\n✅ Sprite texture setter test PASSED")
    sys.exit(0)

# Execute the test after a short delay
import mcrfpy
mcrfpy.setTimer("test", test_sprite_texture_setter, 100)